#ifndef __GCInvoiceData_Customer_h__
#define __GCInvoiceData_Customer_h__

#include <GCInvoiceData/Base.h>

#include <librfcdal/DalDatabase.h>

#include <string>
#include <map>

namespace GCInvoiceData
{
	class GCINVOICEDATASHARED_EXPORT Customer : public Base
	{
		private:
			std::string _strTitle;
			int _nActivity;
			int _nTerms;
			std::string _strContact;
			std::string _strAddress1;
			std::string _strAddress2;
			std::string _strTown;
			std::string _strProvince;
			std::string _strPostCode;
			std::string _strEmail;
			std::string _strPhone;

		public:
			Customer();
			Customer( const Base::Key& key );
			Customer( const Customer& customer );
			virtual ~Customer();

			Customer& operator=( const Customer& customer );

			const std::string& title() const;
			int activity() const;
			int terms() const;
			const std::string& contact() const;
			const std::string& address1() const;
			const std::string& address2() const;
			const std::string& town() const;
			const std::string& province() const;
			const std::string& postCode() const;
			const std::string& email() const;
			const std::string& phone() const;

			void setTitle( const std::string& strTitle );
			void setActivity( int nActivity );
			void setTerms( int nTerms );
			void setContact( const std::string& strContact );
			void setAddress1( const std::string& strAddress1 );
			void setAddress2( const std::string& strAddress2 );
			void setTown( const std::string& strTown );
			void setProvince( const std::string& strProvince );
			void setPostCode( const std::string& strPostCode );
			void setEmail( const std::string& strEmail );
			void setPhone( const std::string& strPhone );

			static Customer fromRow( const RfcDal::DalDatabase::Row& row );
			static Customer create( const Customer& temp );
			void update();
			void remove();
	};

	class GCINVOICEDATASHARED_EXPORT CustomerMap : public std::map<Base::Key, Customer>
	{
		public:
			static CustomerMap fromResult( const RfcDal::DalDatabase::Result& result );
	};
}

GCINVOICEDATASHARED_EXPORT std::ostream& operator<<( std::ostream& s, const GCInvoiceData::Customer& customer );
GCINVOICEDATASHARED_EXPORT std::ostream& operator<<( std::ostream& s, const GCInvoiceData::CustomerMap& customerS );

#endif /* !__GCInvoiceData_Customer_h__ */
