#ifndef __GCInvoiceData_Invoice_h__
#define __GCInvoiceData_Invoice_h__

#include <GCInvoiceData/Base.h>

#include <GCInvoiceData/Customer.h>
#include <GCInvoiceData/Account.h>
#include <GCInvoiceData/Product.h>

#include <librfcdal/DalDatabase.h>

#include <Base4c/Date.h>

#include <string>
#include <map>

namespace GCInvoiceData
{
	class GCINVOICEDATASHARED_EXPORT Invoice : public Base
	{
		public:
			class GCINVOICEDATASHARED_EXPORT Line
			{
				public:
					class GCINVOICEDATASHARED_EXPORT Key
					{
						private:
							int _nInvoice;
							int _nProduct;

						public:
							Key();
							Key( int nInvoice, int nProduct );
							Key( const Key& key );
							virtual ~Key();

							Key& operator=( const Key& key );
							bool operator==( const Key& key ) const;
							bool operator!=( const Key& key ) const;
							bool operator<( const Key& key ) const;

							int invoice() const;
							int product() const;
					};

				private:
					Key _key;
					const Invoice* _pinvoice;
					const Product* _pproduct;
					int _nQuantity;

				public:
					Line();
					Line( const Invoice& invoice, const Product& product );
					Line( const Line& line );
					virtual ~Line();

					Line& operator=( const Line& line );

					const Key& key() const;
					const Invoice& invoice() const;
					const Product& product() const;
					int quantity() const;

					void setQuantity( int nQuantity );

					static Line& fromRow( const RfcDal::DalDatabase::Row& row );
					static Line& create( const Line& temp );
					void update();
					void remove();

					std::ostream& operator<<( std::ostream& s ) const;
			};

			class GCINVOICEDATASHARED_EXPORT LineMap : public std::map<Line::Key, Line>
			{
				public:
					static void fromResult( const RfcDal::DalDatabase::Result& result );

					std::ostream& operator<<( std::ostream& s ) const;
			};


		private:
			const Customer* _pcustomer;
			const Account* _paccount;
			Base4c::Date _dtInvoice;
			LineMap _lineS;

		public:
			Invoice();
			Invoice( const Base::Key& key, const Base4c::Date& dtInvoice );
			Invoice( const Invoice& invoice );
			virtual ~Invoice();

			Invoice& operator=( const Invoice& invoice );

			const Customer& customer() const;
			const Account& account() const;
			const Base4c::Date& invoiceDate() const;
			const LineMap& lines() const;
			LineMap& lines();

			void setCustomer( const Customer& customer );
			void setAccount( const Account& account );
			void setInvoiceDate( const Base4c::Date& dtInvoice );

			static Invoice fromRow( const RfcDal::DalDatabase::Row& row );
			static Invoice create( const Invoice& temp );
			void update();
			void remove();

			std::ostream& operator<<( std::ostream& s ) const;

		protected:
			friend class Line;
	};

	class GCINVOICEDATASHARED_EXPORT InvoiceMap : public std::map<Base::Key, Invoice>
	{
		public:
			static InvoiceMap fromResult( const RfcDal::DalDatabase::Result& result );

			std::ostream& operator<<( std::ostream& s ) const;
	};
}

GCINVOICEDATASHARED_EXPORT std::ostream& operator<<( std::ostream& s, const GCInvoiceData::Invoice::Line& line );
GCINVOICEDATASHARED_EXPORT std::ostream& operator<<( std::ostream& s, const GCInvoiceData::Invoice::LineMap& lineS );
GCINVOICEDATASHARED_EXPORT std::ostream& operator<<( std::ostream& s, const GCInvoiceData::Invoice& invoice );
GCINVOICEDATASHARED_EXPORT std::ostream& operator<<( std::ostream& s, const GCInvoiceData::InvoiceMap& invoiceS );

#endif /* !__GCInvoiceData_Invoice_h__ */
